--	Author:			Arnaud Lagger, arnaud.lagger@epfl.ch
--
--	Version:		pipeline, optimized (fullpins) version of DES algorithm
--

library ieee;

use ieee.std_logic_1164.all;

entity desenc is port
(
	pt	:	in	std_logic_vector(1 TO 64);
	key	:	in	std_logic_vector(1 TO 64);
	ct	:	out	std_logic_vector(1 TO 64);
	reset	:	in	std_logic;
	clk	:	in	std_logic
);
end desenc;

architecture behavior of desenc is
	signal	k1x,k2x,k3x,k4x,k5x,k6x,k7x,k8x,k9x,k10x,k11x,k12x,k13x,k14x,k15x,k16x		:	std_logic_vector(1 to 48);
	signal	l0xa,l1x,l2x,l3x,l4x,l5x,l6x,l7x,l8x,l9x,l10x,l11x,l12x,l13x,l14x,l15x,l16x	:	std_logic_vector(1 to 32);
	signal	r0xa,r1x,r2x,r3x,r4x,r5x,r6x,r7x,r8x,r9x,r10x,r11x,r12x,r13x,r14x,r15x,r16x	:	std_logic_vector(1 to 32);

	component keysched
	port (
		key	:	in	std_logic_vector(1 to 64);
		k1x,k2x,k3x,k4x,k5x,k6x,k7x,k8x,k9x,k10x,k11x,k12x,k13x,k14x,k15x,k16x
			:	out	std_logic_vector(1 to 48)
	);
	end component;

	component ip
	port (
		pt	:	in std_logic_vector(1 TO 64);
		l0x	:	out std_logic_vector(1 TO 32);
		r0x	:	out std_logic_vector(1 TO 32)
	); 
	end component;

	component roundfunc
	port (
		clk		:	in	std_logic;
		reset		:	in	std_logic;
		li,ri	:	in	std_logic_vector(1 to 32);
		k		:	in	std_logic_vector(1 to 48);
		lo,ro	:	out	std_logic_vector(1 to 32)
	);
	end component;

	component fp
	port (
		l,r	:	in	std_logic_vector(1 to 32);
		ct	:	out	std_logic_vector(1 to 64)
	);
	end component;
begin



	keyscheduling:	keysched	port map	(	key=>key,	k1x=>k1x,	k2x=>k2x,	k3x=>k3x,	k4x=>k4x,	k5x=>k5x,	k6x=>k6x,	k7x=>k7x,	k8x=>k8x,	k9x=>k9x,	k10x=>k10x,	k11x=>k11x,	k12x=>k12x,	k13x=>k13x,	k14x=>k14x,	k15x=>k15x,	k16x=>k16x	);

	iperm:			ip 			port map	(	pt=>pt,	l0x=>l0xa,	r0x=>r0xa	);

	round1:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l0xa,	ri=>r0xa,	k=>k1x,	lo=>l1x,	ro=>r1x	);
	round2:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l1x,	ri=>r1x,	k=>k2x,	lo=>l2x,	ro=>r2x	);
	round3:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l2x,	ri=>r2x,	k=>k3x,	lo=>l3x,	ro=>r3x	);
	round4:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l3x,	ri=>r3x,	k=>k4x,	lo=>l4x,	ro=>r4x	);
	round5:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l4x,	ri=>r4x,	k=>k5x,	lo=>l5x,	ro=>r5x	);
	round6:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l5x,	ri=>r5x,	k=>k6x,	lo=>l6x,	ro=>r6x	);
	round7:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l6x,	ri=>r6x,	k=>k7x,	lo=>l7x,	ro=>r7x	);
	round8:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l7x,	ri=>r7x,	k=>k8x,	lo=>l8x,	ro=>r8x	);
	round9:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l8x,	ri=>r8x,	k=>k9x,	lo=>l9x,	ro=>r9x	);
	round10:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l9x,	ri=>r9x,	k=>k10x,	lo=>l10x,	ro=>r10x	);
	round11:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l10x,	ri=>r10x,	k=>k11x,	lo=>l11x,	ro=>r11x	);
	round12:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l11x,	ri=>r11x,	k=>k12x,	lo=>l12x,	ro=>r12x	);
	round13:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l12x,	ri=>r12x,	k=>k13x,	lo=>l13x,	ro=>r13x	);
	round14:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l13x,	ri=>r13x,	k=>k14x,	lo=>l14x,	ro=>r14x	);
	round15:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l14x,	ri=>r14x,	k=>k15x,	lo=>l15x,	ro=>r15x	);
	round16:	roundfunc	port map	(	clk=>clk,	reset=>reset,	li=>l15x,	ri=>r15x,	k=>k16x,	lo=>l16x,	ro=>r16x	);

	fperm:	fp	port map	( l=>r16x,	r=>l16x,	ct=>ct	);

end behavior;