--	Author:			Arnaud Lagger, arnaud.lagger@epfl.ch
--
--	Version:		pipeline, optimized version of DES algorithm
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity test_desenc is end test_desenc;

architecture testbench of test_desenc is

component desenc port
(
	pt		:	in	std_logic_vector(1 TO 16);
	key		:	in	std_logic_vector(1 TO 16);
	ct		:	out	std_logic_vector(1 TO 64);
	reset		:	in	std_logic;
	load_data	:	in	std_logic;
	clk		:	in	std_logic
);
end component;

type test_vector is record
	key	:	std_logic_vector(1 to 16);
	pt	:	std_logic_vector(1 to 16);
	ct	:	std_logic_vector(1 to 64);
end record;

type test_vector_array is array(natural range <>) of test_vector;

constant test_vectors: test_vector_array :=(
	(	key=>x"0000",	pt=>x"0000",	ct=>x"8ca64de9c1b123a7"	),
  	(	key=>x"ffff",	pt=>x"ffff",	ct=>x"7359b2163e4edc58"	),
  	(	key=>x"1111",	pt=>x"1111",	ct=>x"f40379ab9e0ec533"	)
);

	signal   key : std_logic_vector(1 to 16);
	signal   pt : std_logic_vector(1 to 16);
	signal   ct : std_logic_vector(1 to 64);
	signal   clk : std_logic;
	signal   reset : std_logic;
	signal   load_data : std_logic;

begin

dut:	desenc	port map	(     key=>key,     pt=>pt,     ct=>ct,     reset=>reset,	load_data=>load_data,	clk=>clk	);
    
	process
		variable	vector	:	test_vector;
		variable	errors	:	boolean:=false;
	begin
		clk<='1';	wait for 250 ns;	clk<='0';	wait for 250 ns;

		for i in test_vectors'range loop
			vector:=test_vectors(i);
			key<=vector.key;	pt<=vector.pt;
			load_data <= '1';
			clk<='1';	wait for 250 ns;	clk<='0';	wait for 250 ns;
			load_data <= '0';

			for j in 0 to 17 loop clk<='1';	wait for 250 ns;	clk<='0';	wait for 250 ns; end loop;

		end loop;             

		wait;

	end process;

end testbench;
