--	Author:			Arnaud Lagger, arnaud.lagger@epfl.ch
--
--	Version:		state machine, optimized version of DES algorithm
--

library ieee;

use ieee.std_logic_1164.all;

entity control is port
(
	reset		:	in	std_logic;
	clk		:	in	std_logic;
	load_new_pt	:	out	std_logic;
	output_ok	:	out	std_logic;
	load_pt		:	out	std_logic;
	unload_pt	:	out	std_logic;
	load_key	:	out	std_logic;
	unload_key	:	out	std_logic;
	shift		:	out	std_logic_vector(1 to 3)
);
end control;

architecture behavior of control is
	
	type typeetat is (LOAD1, LOAD2, LOAD3, LOAD4, INIT, R1, R2, R3, R4, R5, R6, R7, R8, R9, R10, R11, R12, R13, R14, R15, R16, KEY_LOADING);
	signal etat, etatfutur : typeetat;


begin


	process(etat)
	begin

	case etat is

		when LOAD1 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '1';
			unload_pt <= '0';
			load_key <= '1';
			unload_key <= '0';
			shift <= "000";
			etatfutur <= LOAD2;

		when LOAD2 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '1';
			unload_pt <= '0';
			load_key <= '1';
			unload_key <= '0';
			shift <= "000";
			etatfutur <= LOAD3;

		when LOAD3 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '1';
			unload_pt <= '0';
			load_key <= '1';
			unload_key <= '0';
			shift <= "000";
			etatfutur <= LOAD4;


		when LOAD4 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '1';
			load_key <= '0';
			unload_key <= '1';
			shift <= "000";
			etatfutur <= INIT;

		when INIT =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "011";
			etatfutur <= R1;


		when R1 =>
			load_new_pt <= '1';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "010";
			etatfutur <= R2;

		when R2 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R3;

		when R3 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R4;

		when R4 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R5;

		when R5 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R6;

		when R6 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R7;

		when R7 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R8;

		when R8 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "010";
			etatfutur <= R9;

		when R9 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R10;

		when R10 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R11;

		when R11 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R12;

		when R12 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R13;

		when R13 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '1';
			unload_pt <= '0';
			load_key <= '1';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R14;

		when R14 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '1';
			unload_pt <= '0';
			load_key <= '1';
			unload_key <= '0';
			shift <= "100";
			etatfutur <= R15;

		when R15 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '1';
			unload_pt <= '0';
			load_key <= '1';
			unload_key <= '0';
			shift <= "010";
			etatfutur <= R16;

		when R16 =>
			load_new_pt <= '0';
			output_ok<= '0';
			load_pt <= '0';
			unload_pt <= '1';
			load_key <= '0';
			unload_key <= '1';
			shift <= "000";
			etatfutur <= KEY_LOADING;

		when KEY_LOADING =>
			load_new_pt <= '0';
			output_ok<= '1';
			load_pt <= '0';
			unload_pt <= '0';
			load_key <= '0';
			unload_key <= '0';
			shift <= "011";
			etatfutur <= R1;
			
			
	end case;

	end process;

	process (clk, reset)
	begin

	if (clk'event and clk = '1') then
		etat <= etatfutur;
	end if;

	if(reset='1') then
		etat <= LOAD1;
	end if;

	end process;

end behavior;