--	Author:			Arnaud Lagger, arnaud.lagger@epfl.ch
--
--	Version:		state machine, optimized version of DES algorithm
--

library ieee;

use ieee.std_logic_1164.all;

entity state is port
(
	pt	:	in	std_logic_vector(1 TO 16);
	key	:	in	std_logic_vector(1 TO 16);
	clk	:	in	std_logic;
	reset	:	in	std_logic;
	ct	:	out	std_logic_vector(1 TO 64)
);
end state;

architecture structural of state is

component control
port (
	reset		:	in	std_logic;
	clk		:	in	std_logic;
	load_new_pt	:	out	std_logic;
	output_ok	:	out	std_logic;
	load_pt		:	out	std_logic;
	unload_pt	:	out	std_logic;
	load_key	:	out	std_logic;
	unload_key	:	out	std_logic;
	shift		:	out	std_logic_vector(1 to 3)
);
end component;

component fullround
port (
	pt		:	in	std_logic_vector(1 TO 64);
	xkey		:	in	std_logic_vector(1 TO 48);
	reset		:	in	std_logic;
	clk		:	in	std_logic;
	load_new_pt	:	in	std_logic;
	output_ok	:	in	std_logic;
	ct		:	out	std_logic_vector(1 TO 64)
);
end component;

component keysched
port (
	the_key	:	in	std_logic_vector(1 to 64);
	shift	:	in	std_logic_vector(1 to 3);
	clk	:	in	std_logic;
	ki	:	out	std_logic_vector(1 to 48)
);
end component;

component converter
port (
	input	:	in std_logic_vector(1 to 16);
	load	:	in std_logic;
	unload	:	in std_logic;
	clk	:	in std_logic;
	output	:	out std_logic_vector (1 to 64)
);  
end component;

signal load_new_pt	:		std_logic;
signal output_ok	:		std_logic;
signal load_pt_sig	:		std_logic;
signal unload_pt_sig	:		std_logic;
signal load_key_sig	:		std_logic;
signal unload_key_sig	:		std_logic;
signal shift_sig	:		std_logic_vector(1 to 3);
signal ki_sig		:		std_logic_vector(1 to 48);
signal key_sig		:		std_logic_vector(1 to 64);
signal pt_sig		:		std_logic_vector(1 to 64);


begin

control_unit	: control	port map ( reset=>reset, clk=>clk, load_new_pt=>load_new_pt, output_ok=>output_ok, load_pt=>load_pt_sig, unload_pt=>unload_pt_sig, load_key=>load_key_sig, unload_key=>unload_key_sig, shift=>shift_sig );

converter_pt	: converter	port map ( input=>pt, load=>load_pt_sig, unload=>unload_pt_sig, clk=>clk, output=>pt_sig );
converter_key	: converter	port map ( input=>key, load=>load_key_sig, unload=>unload_key_sig, clk=>clk, output=>key_sig );

datapath	: fullround	port map ( pt=>pt_sig, xkey=>ki_sig, reset=>reset, clk=>clk,load_new_pt=>load_new_pt, output_ok=>output_ok, ct=>ct );

key_proc	: keysched	port map ( the_key=>key_sig, shift=>shift_sig, clk=>clk, ki=>ki_sig );

end structural;