--	Author:			Arnaud Lagger, arnaud.lagger@epfl.ch
--
--	Version:		state machine, optimized version of DES algorithm
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity test_fpga is end test_fpga;

architecture testbench of test_fpga is

component state port
(
	pt	:	in	std_logic_vector(1 TO 16);
	key	:	in	std_logic_vector(1 TO 16);
	clk	:	in	std_logic;
	reset	:	in	std_logic;
	ct	:	out	std_logic_vector(1 TO 64)
);
end component;

type test_vector is record
	key	:	std_logic_vector(1 to 16);
	pt	:	std_logic_vector(1 to 16);
	ct	:	std_logic_vector(1 to 64);
end record;

type test_vector_array is array(natural range <>) of test_vector;

constant test_vectors: test_vector_array :=(
  	(	key=>x"1111",	pt=>x"1111",	ct=>x"f40379ab9e0ec533"	),
  	(	key=>x"ffff",	pt=>x"0000",	ct=>x"caaaaf4deaf1dbae"	),
	(	key=>x"0000",	pt=>x"0000",	ct=>x"8ca64de9c1b123a7"	)
);

	signal   key : std_logic_vector(1 to 16);
	signal   pt : std_logic_vector(1 to 16);
	signal   ct : std_logic_vector(1 to 64);
	signal   clk : std_logic;
	signal	 reset : std_logic;

begin

dut:	state	port map	(	pt=>pt,  key=>key,    ct=>ct,  reset=>reset,     clk=>clk	);
    
	process
		variable	vector	:	test_vector;
	begin

		key<=x"0000";	pt<=x"0000";

		for j in 0 to 17 loop	clk<='0';	wait for 250 ns;	clk<='1';	wait for 250 ns;	end loop;

		key<=x"ffff";	pt<=x"ffff";
		for j in 0 to 3 loop	clk<='0';	wait for 250 ns;	clk<='1';	wait for 250 ns;	end loop;

		for i in test_vectors'range loop

			for j in 0 to 16 loop	clk<='0';	wait for 250 ns;	clk<='1';	wait for 250 ns;	end loop;
			vector:=test_vectors(i);
			key<=vector.key;	pt<=vector.pt;
			for j in 0 to 3 loop	clk<='0';	wait for 250 ns;	clk<='1';	wait for 250 ns;	end loop;
		end loop;
		
		for j in 0 to 17 loop	clk<='0';	wait for 250 ns;	clk<='1';	wait for 250 ns;	end loop;

		wait;

	end process;

end testbench;
